#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "stx7105.h"
#include "stx7105_fdma.h"
#include "stx7105_fdma_fw.h"
#include "stx7105_utils.h"

#define LED_RED_GPIO PIO0
#define LED_RED_PIN  5U

#define LED_BLUE_GPIO PIO0
#define LED_BLUE_PIN  4U

#define CONSOLE_ASC     ASC2
#define SYSTEM_DEVID    (0xFE001000U) /* DEVID */
#define SYSTEM_CONFIG34 (0xFE001188U) /* PIO4 */
#define SYSTEM_CONFIG7  (0xFE00111CU) /* RXSEL */

#define MEMTEST_START 0x82000000
#define MEMTEST_END   0x8F000000

#define DMA_BUFFER_SIZE 16
uint8_t src_buffer[DMA_BUFFER_SIZE];

void uart_init(void) {
    PIO4->CLR_PC0 = 1U; /* PC = 110, AFOUT, PP */
    PIO4->SET_PC1 = 1U;
    PIO4->SET_PC2 = 1U;

    *(uint32_t *)SYSTEM_CONFIG34 = 0x00000100UL;    /* BIT[8,0] = 10, AF 3 */
    *(uint32_t *)SYSTEM_CONFIG7 &= ~(0x00000006UL); /* BIT[2:1], UART2 RX SEL */

    CONSOLE_ASC->CTRL     = 0x1509UL; /* 8N1, RX enable, FIFO enable, Baud mode 1 */
    CONSOLE_ASC->BAUDRATE = 0x04B8UL; /* 115200 in baud mode 1, assuming Fcomm=100MHz */
    CONSOLE_ASC->TX_RST   = 0x01UL;   /* Reset TX FIFO, any value OK */
    CONSOLE_ASC->RX_RST   = 0x01UL;   /* Reset RX FIFO, any value OK */
    CONSOLE_ASC->CTRL     = 0x1589UL; /* 8N1, RX enable, FIFO enable, Baud mode 1 */
}

static void configure_fdma0(void) {
    FDMA0->SLIM_CLK_GATE |= (FDMA_SLIM_CLK_GATE_DIS_Msk | FDMA_SLIM_CLK_GATE_RESET_Msk);
    FDMA0->PERIPH_STBUS_SYNC |= FDMA_PERIPH_STBUS_SYNC_DIS_Msk;

    for (uint32_t i = 0; i < stx7105_fdma0_fw.text_size; i++) {
        FDMA0->SLIM_IMEM[i] = stx7105_fdma0_fw.text[i];
    }

    for (uint32_t i = 0; i < stx7105_fdma0_fw.data_size; i++) {
        FDMA0->SLIM_DMEM[i] = stx7105_fdma0_fw.data[i];
    }

    FDMA0->SLIM_CLK_GATE &= ~FDMA_SLIM_CLK_GATE_DIS_Msk;
    FDMA0->PERIPH_INT_CLR = 0xFFFFFFFFU;
    FDMA0->PERIPH_CMD_CLR = 0xFFFFFFFFU;

    FDMA0->SLIM_EN |= FDMA_SLIM_EN_RUN_Msk;

    printf("FDMA0 SLIM ID: 0x%08lx\r\n", FDMA0->SLIM_ID);
    printf("FDMA0 SLIM Version: 0x%08lx\r\n", FDMA0->SLIM_VER);
}

static void configure_fdma1(void) {
    FDMA1->SLIM_CLK_GATE |= (FDMA_SLIM_CLK_GATE_DIS_Msk | FDMA_SLIM_CLK_GATE_RESET_Msk);
    FDMA1->PERIPH_STBUS_SYNC |= FDMA_PERIPH_STBUS_SYNC_DIS_Msk;

    for (uint32_t i = 0; i < stx7105_fdma1_fw.text_size; i++) {
        FDMA1->SLIM_IMEM[i] = stx7105_fdma1_fw.text[i];
    }

    for (uint32_t i = 0; i < stx7105_fdma1_fw.data_size; i++) {
        FDMA1->SLIM_DMEM[i] = stx7105_fdma1_fw.data[i];
    }

    FDMA1->SLIM_CLK_GATE &= ~FDMA_SLIM_CLK_GATE_DIS_Msk;
    FDMA1->PERIPH_INT_CLR = 0xFFFFFFFFU;
    FDMA1->PERIPH_CMD_CLR = 0xFFFFFFFFU;

    FDMA1->SLIM_EN |= FDMA_SLIM_EN_RUN_Msk;

    printf("FDMA1 SLIM ID: 0x%08lx\r\n", FDMA1->SLIM_ID);
    printf("FDMA1 SLIM Version: 0x%08lx\r\n", FDMA1->SLIM_VER);
}

int main(void) {
    init_led(LED_RED_GPIO, LED_RED_PIN, 0U);
    init_led(LED_BLUE_GPIO, LED_BLUE_PIN, 0U);

    setbuf(stdout, NULL);
    setbuf(stderr, NULL);

    uart_init();

    printf("Hello world\r\n");

    printf("CKGA Opts 1: 0x%08lx\r\n", CKGA->CLKOPSRC_SWITCH_CFG);
    printf("CKGA Opts 2: 0x%08lx\r\n", CKGA->CLKOPSRC_SWITCH_CFG2);

    configure_fdma0();
    configure_fdma1();

    FDMA_FWRegs_TypeDef *fdma0_fwregs = (FDMA_FWRegs_TypeDef *)(FDMA0->SLIM_DMEM);

    fdma0_fwregs->CHANNEL[0].NEXT   = (uint32_t)NULL;
    fdma0_fwregs->CHANNEL[0].NBYTES = DMA_BUFFER_SIZE;
    fdma0_fwregs->CHANNEL[0].LENGTH = DMA_BUFFER_SIZE;
    fdma0_fwregs->CHANNEL[0].SADDR  = (uint32_t)src_buffer;
    fdma0_fwregs->CHANNEL[0].DADDR  = CONSOLE_ASC->TX_BUF;
    fdma0_fwregs->CHANNEL[0].S_STRIDE = 0U;
    fdma0_fwregs->CHANNEL[0].D_STRIDE = 0U;
    fdma0_fwregs->REQ_CTRL[0]       = (1U << FDMA_FwRegs_REQ_CTLN_NUM_OPS_Pos) | FDMA_FwRegs_REQ_CTLN_INC_ADDR_Msk;

    delay_ms(5000);

    for (;;) {
        set_led(LED_BLUE_GPIO, LED_BLUE_PIN, 1U);
        delay_ms(500);
        set_led(LED_BLUE_GPIO, LED_BLUE_PIN, 0U);
        delay_ms(500);
    }

    return 0;
}
